% -------------------------------------------------------------------
% ckbs: Constrained Kalman-Bucy Smoother Program: Copyright (C) 2006
% Authors: Bradlely Bell:        bradbell at washington dot edu
%          Gianluigi Pillonetto: giapi at dei dot unipd dot it
% License: GNU General Public License Version 2
% -------------------------------------------------------------------
% $begin vanderpol_g.m$$ $newlinech %$$ 
% $spell
%	Jacobian
%	ckbs
%	gk
%	xk
%	est
%	mu
%	vanderpol
%	dt
%	xi
% $$
% $section ckbs_nonlinear: Vanderpol Transition Model Mean Example$$
%
% $index dynamics, vanderpol$$
% $index transition, vanderpol$$
% $index vanderpol, transition$$
% $index ckbs_nonlinear, transition$$
% $index nonlinear, transition$$.
%
% $head Syntax$$
% $codei%global vanderpol_g_info
% %$$
% $codei%[%gk%] %  %  = vanderpol_g(%k%, %xk1%)
% %$$
% $codei%[%gk%, %Gk%] = vanderpol_g(%k%, %xk1%)
% %$$
%
% $head Notation$$
% $codei%
%	%initial%  = vanderpol_g_info.initial
%	%dt%       = vanderpol_g_info.dt
%	%mu%       = vanderpol_g_info.mu
% %$$
%
% $head Purpose$$
% Computes $icode gk$$ as the mean of the state at time index $icode k$$ given
% $icode xk1$$ is its value at time index $icode%k%-1%$$.
% This mean is Euler's discrete approximation for the solution of the 
% Van Der Pol oscillator ODE; i.e.,
% $latex \[
% \begin{array}{rcl}
%	x_1 '(t) & = & x_2 (t) 
%	\\
%	x_2 '(t) & = & \mu [ 1 - x_1(t)^2 ] x_2 (t) - x_1(t)	
% \end{array}
% \] $$
% where the initial state is $icode xk1$$ and the time step is $icode dt$$.
%
% $head initial$$
% is a column vector of length two specifying the initial estimate
% for the state vector at time index one.
%
% $head dt$$
% is a scalar specifying the time between points for this Kalman smoother
% (also referred to as $latex \Delta t$$ below).
%
% $head mu$$
% is a scalar specifying the value of $latex \mu$$ in the Van Der Pol ODE. 
%
% $head k$$
% is a positive integer scalar specifying the current time index
% (not used).
%
% $head xk1$$
% is a column vector with two elements specifying a value for
% the state vector at the previous time index $icode%k%-1%$$.
%
% $head gk$$
% If $icode%k% == 1%$$, 
% the return value $icode gk$$ is two element column vector
% equal to $icode initial$$.
% Otherwise, $icode gk$$ is the two element column vector given by
% $latex \[
% \begin{array}{rcl}
%	g_{1,k} & = & x_{1,k-1} + x_{2,k-1} \Delta t 
%	\\
%	g_{2,k} & = & x_{2,k-1} 
%	          + [ \mu ( 1 - x_{1,k-1}^2 ) x_{2,k-1} - x_{1,k-1} ] \Delta t
% \end{array}
% \] $$
% where $latex ( g_{1,k} , g_{2,k} )^\R{T}$$ is $icode gk$$ and
% $latex ( x_{1,k-1} , x_{2,k-1} )^\R{T}$$ is $icode xk1$$.
%
% $head Gk$$
% The return value $icode Gk$$ is an $icode%n% x %n%$$ matrix equal to the 
% Jacobian of $icode gk$$ w.r.t $icode xk1$$. 
%
% $head Source Code$$
% $newlinech $$ $codep
function [gk, Gk] = vanderpol_g(k, xk1)
	global vanderpol_g_info
	initial = vanderpol_g_info.initial; 
	dt      = vanderpol_g_info.dt; 
	mu      = vanderpol_g_info.mu;
	n       = 2;
	if (size(xk1,1) ~= n) | (size(xk1,2) ~= 1)
		size_xk1_1 = size(xk1, 1)
		size_xk1_2 = size(xk1, 2)
		error('vanderpol_g: xk1 not a column vector with two elements')
	end
	if (size(initial,1) ~= n) | (size(initial,2) ~= 1)
		size_initial_1 = size(initial, 1)
		size_initial_2 = size(initial, 2)
		error('vanderpol_g: initial not a column vector with two elements')
	end
	if (size(dt,1)*size(dt,2) ~= 1) | (size(mu,1)*size(mu,2) ~= 1)
		size_dt_1 = size(dt, 1)
		size_dt_2 = size(dt, 2)
		size_mu_1 = size(mu, 1)
		size_mu_2 = size(mu, 2)
		error('vanderpol_g: dt or mu is not a scalar')
	end
	if k == 1 
		gk = initial;
		Gk = zeros(n, n);
	else
		gk      = zeros(n, 1);
		gk(1)   = xk1(1) + xk1(2) * dt;
		gk(2)   = xk1(2) + (mu * (1 - xk1(1)*xk1(1)) * xk1(2) - xk1(1)) * dt;
		%
		Gk      = zeros(n, n);
		Gk(1,1) = 1;
		Gk(1,2) = dt;
		Gk(2,1) = (mu * (- 2 * xk1(1)) * xk1(2) - 1) * dt;
		Gk(2,2) = 1 + mu * (1 - xk1(1)*xk1(1)) * dt;
	end
	return
end
% $$ $newlinech %$$
% $end
